require("spec_helper.js");
require("../jquery.snazzyselect.js");

Screw.Unit(function(){
  describe("Snazzy Select", function(){
    before(function() { 
      var fixtures = $('#fixtures').empty();
      $('ul.snazzy_select').remove();
      form = $('<form />').appendTo(fixtures);
      var container = $('<div />').appendTo(form);
      div = $('<div />').appendTo(container).addClass('snazzy_selected');
      select = $('<select />').appendTo(container).append('<option class="awesome">Everyone</option>').append("<option selected='selected' value='sean'>Sean Cashin</option>").append("<option value='ryan'>Ryan Sonnek</option>").snazzySelect();
      snazzySelect = $('ul.snazzy_select');
      value = div.find('.value');
    });
    it("automatically hides the select input", function(){
      expect(select.is(":hidden")).to(equal, true);
    });
    it("initializes div with text from selected option", function(){
      expect(value.html()).to(equal, 'Sean Cashin');
    });
    it('automatically creates a hidden unordered list with options from select input', function() {
      expect(snazzySelect.size()).to(equal, 1);
      expect(snazzySelect.is(':hidden')).to(equal, true);
      expect(snazzySelect.find('li').size()).to(equal, 3);
    });
    it("preserves class attribute from option to list element", function(){
      expect(snazzySelect.find('li.awesome').size()).to(equal, 1);
    });
    it('fires change listener when element selected', function() {
      var changed = false;
      select.change(function() {
        changed = true;
      });
      value.click();
      snazzySelect.find('li:last').click();
      expect(changed).to(equal, true);
    });

    describe("clicking on the current value", function() {
      before(function() {
        value.click();
      });
      it('shows the snazzy list', function(){
        expect(snazzySelect.is(":visible")).to(equal, true);
      });
      it('adds hover class to current selection', function(){
        expect(snazzySelect.find('li.snazzy_hover').html()).to(equal, 'Sean Cashin');
      });
    });

    describe("clicking on the dropdown icon", function() {
      before(function() {
        div.find('.icon').click();
      });
      it('shows the snazzy list', function(){
        expect(snazzySelect.is(":visible")).to(equal, true);
      });
    });

    describe("clicking on the current value twice", function() {
      before(function() {
        value.click();
        value.click();
      });
      it('hides the snazzy list', function(){
        expect(snazzySelect.is(":visible")).to(equal, false);
      });
    });

    describe("clicking on another element after dropdown is visible", function() {
      before(function() {
        value.click();
        $('#some_other_text').click();
      });
      it('hides the snazzy list', function(){
        expect(snazzySelect.is(":visible")).to(equal, false);
      });
    });

    describe("selecting element from list via mouse click", function() {
      before(function(){
        value.click();
        selected = snazzySelect.find('li:last');
        selected.click();
      });
      it('updates text of current value to match text of selected element', function(){
        expect(value.html()).to(equal, selected.text());
      });
      it('hides snazzy list', function(){
        expect(snazzySelect.is(":hidden")).to(equal, true);
      });
    });

    describe('keyboard navigation', function() {
      it('down arrow hovers over next result', function() {
        value.click();
        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.DOWN;
        e.keyCode = $.SnazzySelect.KEYS.DOWN;
      	value.trigger(e);

        expect(snazzySelect.find('li.snazzy_hover').text()).to(equal, 'Ryan Sonnek');
      });
      it('up arrow hovers over previous result', function() {
        value.click();
        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.UP;
        e.keyCode = $.SnazzySelect.KEYS.UP;
      	value.trigger(e);

        expect(snazzySelect.find('li.snazzy_hover').text()).to(equal, 'Everyone');
      });
      it('escape key hides the snazzy select dropdown', function() {
        value.click();
        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.ESCAPE;
        e.keyCode = $.SnazzySelect.KEYS.ESCAPE;
      	value.trigger(e);

        expect(snazzySelect.is(':hidden')).to(equal, true);
      });
      it('return selects highlighted element', function() {
        value.click();
        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.UP;
        e.keyCode = $.SnazzySelect.KEYS.UP;
      	value.trigger(e);

        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.RETURN;
        e.keyCode = $.SnazzySelect.KEYS.RETURN;
      	value.trigger(e);

        expect(value.html()).to(equal, 'Everyone');
      });
      it('tab selects highlighted element', function() {
        value.click();
        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.UP;
        e.keyCode = $.SnazzySelect.KEYS.UP;
      	value.trigger(e);

        e = $.Event("keydown");
        e.which = $.SnazzySelect.KEYS.TAB;
        e.keyCode = $.SnazzySelect.KEYS.TAB;
      	value.trigger(e);

        expect(value.html()).to(equal, 'Everyone');
      });
    });

    describe("resetting the form", function() {
      before(function() {
        value.click();
        selected = snazzySelect.find('li:last');
        selected.click();
        form.each(function(){
          this.reset();
      	});
      });
      it('resets the selected text after event triggered', function() {
        //this test fails because the assertion runs before the reset handler
        //need to find a way to run this test after the setTimeout callback fires
        //expect(value.html()).to(equal, 'Everyone');
      });
    });

    describe('extra elements', function() {
      before(function() { 
        var fixtures = $('#fixtures').empty();
        $('ul.snazzy_select').remove();
        var container = $('<div />').appendTo(fixtures);
        div = $('<div />').appendTo(container).addClass('snazzy_selected');
        extraLink = $("<a href='http://google.com'>google it</a>");
        select = $('<select />').appendTo(container).append('<option>Everyone</option>').append("<option selected='selected' value='sean'>Sean Cashin</option>").append("<option value='ryan'>Ryan Sonnek</option>").snazzySelect({
          extraElements: extraLink
        });
        snazzySelect = $('ul.snazzy_select');
      });
      it('adds placeholder break element after options', function() {
        expect(snazzySelect.children('li').eq(3).hasClass('snazzy_break')).to(equal, true);
      });
      it('adds content to end of list', function() {
        expect(snazzySelect.children('li').eq(4).find('a').attr('href')).to(equal, extraLink.attr('href'));
      });
    });
  });
});
